<?php
if (!defined('__TYPECHO_ROOT_DIR__')) exit;
/**
 * 浏览数、点赞数、热门文章统计插件。基于<a href="http://lixianhua.com" target="_blank">StatPro</a>和<a href="http://kan.willin.org/typecho/" target="_blank">Views</a>插件开发
 * 
 * @package StatPro
 * @author YoviSun
 * @version 1.0
 * @link http://www.yovisun.com
 */
class StatPro_Plugin implements Typecho_Plugin_Interface
{
	public static $info = array();
	public static $mem = array();
    /**
     * 激活插件方法,如果激活失败,直接抛出异常
     * 
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function activate()
    {
        $db = Typecho_Db::get();
        $prefix = $db->getPrefix();
        // contents 表中若无 viewsNum 字段则添加
        if (!array_key_exists('viewsNum', $db->fetchRow($db->select()->from('table.contents'))))
            $db->query('ALTER TABLE `'. $prefix .'contents` ADD `viewsNum` INT(10) DEFAULT 0;');
		// contents 表中若无 likesNum 字段则添加
        if (!array_key_exists('likesNum', $db->fetchRow($db->select()->from('table.contents'))))
            $db->query('ALTER TABLE `'. $prefix .'contents` ADD `likesNum` INT(10) DEFAULT 0;');
        //增加浏览数
        Typecho_Plugin::factory('Widget_Archive')->singleHandle = array('StatPro_Plugin', 'viewCounter');
        //把新增的字段添加到查询中
        Typecho_Plugin::factory('Widget_Archive')->select = array('StatPro_Plugin', 'selectHandle');
		//添加动作
		Helper::addAction('likes', 'StatPro_Action');
		
		Typecho_Plugin::factory('Widget_Archive')->header = array('StatPro_Plugin','insertCss');
		Typecho_Plugin::factory('Widget_Archive')->footer = array('StatPro_Plugin','insertJs');
		
		Typecho_Plugin::factory('index.php')->begin = array('StatPro_Plugin', 'setBegin');
        Typecho_Plugin::factory('index.php')->end = array('StatPro_Plugin', 'setEnd');
		
    }
    
    /**
     * 禁用插件方法,如果禁用失败,直接抛出异常
     * 
     * @static
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function deactivate()
	{
		Helper::removeAction('likes');
		
        $delFields = Typecho_Widget::widget('Widget_Options')->plugin('StatPro')->delFields;
        if($delFields){
            $db = Typecho_Db::get();
            $prefix = $db->getPrefix();
            $db->query('ALTER TABLE `'. $prefix .'contents` DROP `viewsNum`;');
			$db->query('ALTER TABLE `'. $prefix .'contents` DROP `likesNum`;');
        }
	}
    
    /**
     * 获取插件配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form 配置面板
     * @return void
     */
    public static function config(Typecho_Widget_Helper_Form $form)
	{	
		$delFields = new Typecho_Widget_Helper_Form_Element_Radio('delFields', 
            array(0=>_t('保留数据'),1=>_t('删除数据'),), '0', _t('卸载设置'),_t('卸载插件后数据是否保留'));
        $form->addInput($delFields);
		$allow_stat = new Typecho_Widget_Helper_Form_Element_Radio('allow_stat', 
            array(0=>_t('关闭'),1=>_t('开启'),), '1', _t('统计信息'),_t('是否开启统计功能'));
        $form->addInput($allow_stat);
		$allow_stat_mem = new Typecho_Widget_Helper_Form_Element_Radio('allow_stat_mem', 
            array(0=>_t('关闭'),1=>_t('开启'),), '0', _t('统计内存开销'),_t('是否开启内存开销统计'));
        $form->addInput($allow_stat_mem);
	}
    
    /**
     * 个人用户的配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form
     * @return void
     */
    public static function personalConfig(Typecho_Widget_Helper_Form $form){}
    
	//记录开始执行的参数
	public static function setBegin(){
		$options = Typecho_Widget::widget('Widget_Options')->plugin('StatPro');
		
		if(!$options->allow_stat) return;
		
		self::$info['begin'] = microtime(TRUE);
		
		if($options->allow_stat_mem)
			self::$mem['begin'] =  memory_get_usage();
	}

	//记录执行结束的参数
	public static function setEnd(){
		$options = Typecho_Widget::widget('Widget_Options')->plugin('StatPro');
		
		if(!$options->allow_stat) return;
		
		if(isset(self::$info['end'])) return;
		
		self::$info['end'] = microtime(TRUE);
		
		if($options->allow_stat_mem)
			self::$mem['end'] =  memory_get_usage();
		
	}


	//显示记录的各种统计项
	public static function runtime($format='吞吐率:{rate},运行时间:{runtime},内存开销:{mem},加载文件:{files}'){
		$options = Typecho_Widget::widget('Widget_Options');
		if(!isset($options->plugins['activated']['StatPro'])) return;

		$options = $options->plugin('StatPro');
		if(!$options->allow_stat) return;
		
		if(isset(self::$info['begin']) && !isset(self::$info['end'])){
			self::setEnd();
		}
		$stat = array(
			'runtime'=>'未统计',
			'mem'=>'未统计',
			'files'=>count(get_included_files()),
			'rate'=>'未统计',
		);
		
		if(isset(self::$info['begin']) && isset(self::$info['end'])){
			$stat['runtime'] = number_format(self::$info['end']-self::$info['begin'],4).' s';
			$stat['rate'] = number_format(1/(self::$info['end']-self::$info['begin']),2).' req/s';
		}
		if(isset(self::$mem['begin']) && isset(self::$mem['end'])){
			$stat['mem'] = number_format((self::$mem['end']-self::$mem['begin'])/1024).' kb';
		}
		echo str_replace(array('{runtime}','{mem}','{files}','{rate}'),$stat,$format);
	}

    /**
     * 获得访问次数
     *
     * 语法: StatPro_Plugin::getViewsNum();
     *
     *
     * @access public
     * @return int
     */
    public static function getViewsNum()
    {
        $db = Typecho_Db::get();
        $cid = Typecho_Widget::widget('Widget_Archive')->cid;
        $row = $db->fetchRow($db->select('viewsNum')->from('table.contents')->where('cid = ?', $cid));
        return number_format($row['viewsNum']);
    }


    /**
     * 获得点赞次数
     *
     * 语法: StatPro_Plugin::getlikesNum();
     *
     * @access public
     * @return int
     */
    public static function getlikesNum()
    {
        $db = Typecho_Db::get();
        $cid = Typecho_Widget::widget('Widget_Archive')->cid;
        $row = $db->fetchRow($db->select('likesNum')->from('table.contents')->where('cid = ?', $cid));
        return number_format($row['likesNum']);
    }


    /**
     * 增加浏览量
     * @params Widget_Archive   $archive
     * @return void
     */
    public static function viewCounter($archive){
        if($archive->is('single')){
            $cid = $archive->cid;
            $views = Typecho_Cookie::get('__post_views');
            if(empty($views)){
                $views = array();
            }else{
                $views = explode(',', $views);
            }
            if(!in_array($cid,$views)){
                $db = Typecho_Db::get();
                $db->query($db->update('table.contents')->rows(array('viewsNum' => (int)$archive->viewsNum+1))->where('cid = ?', $cid));
                array_push($views, $cid);
                $views = implode(',', $views);
                Typecho_Cookie::set('__post_views', $views); //记录查看cookie
            }
        }
    }

	public static function insertCss($header,$widget){
        if($widget->is("single")){ 

		$action = Typecho_Common::url('/action/',Helper::options()->index);
		echo '<style type="text/css">.StatPro-dialog{position:fixed;top:100px;left:50%;padding:10px;background-color:#fff;display:none;-webkit-border-radius: 3px;-moz-border-radius: 3px;border-radius: 3px;z-index:1024;}
.StatPro-dialog.error{background-color:#f40;color:#fff;}
.StatPro-dialog.success{background-color:#24AA42;color:#fff;}</style><script type="text/javascript">window.action="'.$action.'";</script>';
}
	}

	public static function insertJs($widget){
        if($widget->is("single")){ 

		$script = <<<EOT
<script type="text/javascript">
$(function(){
	$('.btn-like').click(function(e){
		e.stopPropagation();
		e.preventDefault();		
		var that = $(this),num = $(this).data('num'), cid = $(this).data('cid'),numEl = that.find('.post-likes-num');
		if(cid===undefined) return false;
		$('.post-likes-num').html('<i class="fa fa-spinner fa-pulse"></i>');
		$.get(window.action+'likes?cid='+cid).success(function(rs){
			if(rs.status==1){
				if(numEl.length>0){
					numEl.text(num+1);
				}
				StatProAlert(rs.msg===undefined ? '已成功为该文章点赞!' : rs.msg);
				$('.post-likes-num').text(num+1);
			}else{
				$('.post-likes-num').text(num);
				StatProAlert(rs.msg===undefined ? '操作出错!' : rs.msg,'err');
			}
		});
	});
});
function StatProAlert(msg,type,time){
	type = type === undefined ? 'success' : 'error';
	time = time === undefined ? (type=='success' ? 1500 : 3000) : time;
	var html = '<div class="StatPro-dialog '+type+'">'+msg+'</div>';
	$(html).appendTo($('body')).fadeIn(300,function(){
		setTimeout(function(){
			$('body > .StatPro-dialog').remove();
		},time);
	});
}
</script>
EOT;

		echo $script;


         }

	}

    //cleanAttribute('fields')清除查询字段，select * 
    public static function selectHandle($archive){
        $user = Typecho_Widget::widget('Widget_User');
		if ('post' == $archive->parameter->type || 'page' == $archive->parameter->type) {
            if ($user->hasLogin()) {
                $select = $archive->select()->where('table.contents.status = ? OR table.contents.status = ? OR
                        (table.contents.status = ? AND table.contents.authorId = ?)',
                        'publish', 'hidden', 'private', $user->uid);
            } else {
                $select = $archive->select()->where('table.contents.status = ? OR table.contents.status = ?',
                        'publish', 'hidden');
            }
        } else {
            if ($user->hasLogin()) {
                $select = $archive->select()->where('table.contents.status = ? OR
                        (table.contents.status = ? AND table.contents.authorId = ?)', 'publish', 'private', $user->uid);
            } else {
                $select = $archive->select()->where('table.contents.status = ?', 'publish');
            }
        }
        $select->where('table.contents.created < ?', Typecho_Date::gmtTime());
        $select->cleanAttribute('fields');
        return $select;
	}


	/**
     * 输出最热门的文章
     *
     * 热度值根据文章阅读数、点赞数、评论数获得，分别对每个赋予权值
     *
     * 语法: StatPro_Plugin::mostHoted();
     *
     * @access public
     * @param int     $limit  文章数目
     * @param string  $shownum 是否显示浏览数量
     * @param string  $before 前字串
     * @param string  $after  后字串
     * @return string
     */
    public static function mostHoted($limit = 10, $shownum = true, $before = '<br/> - ( 访问: ', $after = ' 次 ) ')
    {
        $db = Typecho_Db::get();
        $options = Typecho_Widget::widget('Widget_Options');
        $limit = is_numeric($limit) ? $limit : 10;
        $posts = $db->fetchAll($db->query("select * from ".$db->getPrefix()."contents where type = 'post' AND status = 'publish' AND password IS NULL order by viewsNum*1+likesNum*2+commentsNum*3 desc limit ".$limit));
        if ($posts) {
            foreach ($posts as $post) {
                $result = Typecho_Widget::widget('Widget_Abstract_Contents')->push($post);
                $post_viewsNum = number_format($result['viewsNum']);
                $post_likesNum = number_format($result['likesNum']);
                $post_commentsNum = number_format($result['commentsNum']);
                $post_hotVal = $post_viewsNum*1+$post_likesNum*2+$post_commentsNum*3;
                $post_title = htmlspecialchars($result['title']);
                $permalink = $result['permalink'];
                if($shownum == true){
                	echo "<li><a href='$permalink' title='$post_title'>$post_title</a><span style='font-size:70%'>$before $post_viewsNum $after</span></li>\n";
              	}else{
              		echo "<li><a href='$permalink' title='$post_title &#13;热度值：$post_hotVal'>$post_title</a></li>\n";
              	}
            }

        } else {
            echo "<li>N/A</li>\n";
        }
    }


	/**
     * 输出访问最多的文章
     *
     * 语法: StatPro_Plugin::mostViewed();
     *
     * @access public
     * @param int     $limit  文章数目
     * @param string  $shownum 是否显示浏览数量
     * @param string  $before 前字串
     * @param string  $after  后字串
     * @return string
     */
    public static function mostViewed($limit = 10, $shownum = true, $before = '<br/> - ( 阅读: ', $after = ' 次 ) ')
    {
        $db = Typecho_Db::get();
        $options = Typecho_Widget::widget('Widget_Options');
        $limit = is_numeric($limit) ? $limit : 10;
        $posts = $db->fetchAll($db->select()->from('table.contents')
                 ->where('type = ? AND status = ? AND password IS NULL', 'post', 'publish')
                 ->order('viewsNum', Typecho_Db::SORT_DESC)
                 ->limit($limit)
                 );

        if ($posts) {
            foreach ($posts as $post) {
                $result = Typecho_Widget::widget('Widget_Abstract_Contents')->push($post);
                $post_viewsNum = number_format($result['viewsNum']);
                $post_title = htmlspecialchars($result['title']);
                $permalink = $result['permalink'];
                if($shownum == true){
                	echo "<li><a href='$permalink' title='$post_title'>$post_title</a><span style='font-size:70%'>$before $post_viewsNum $after</span></li>\n";
              	}else{
              		echo "<li><a href='$permalink' title='$post_title &#13;阅读数：$post_viewsNum'>$post_title</a></li>\n";
              	}
            }

        } else {
            echo "<li>N/A</li>\n";
        }
    }

    /**
     * 输出点赞最多的文章
     *
     * 语法: StatPro_Plugin::mostLiked();
     *
     * @access public
     * @param int     $limit  文章数目
     * @param string  $shownum 是否显示浏览数量
     * @param string  $before 前字串
     * @param string  $after  后字串
     * @return string
     */
    public static function mostLiked($limit = 10, $shownum = true, $before = '<br/> - ( 点赞: ', $after = ' 次 ) ')
    {
        $db = Typecho_Db::get();
        $options = Typecho_Widget::widget('Widget_Options');
        $limit = is_numeric($limit) ? $limit : 10;
        $posts = $db->fetchAll($db->select()->from('table.contents')
                 ->where('type = ? AND status = ? AND password IS NULL', 'post', 'publish')
                 ->order('likesNum', Typecho_Db::SORT_DESC)
                 ->limit($limit)
                 );

        if ($posts) {
            foreach ($posts as $post) {
                $result = Typecho_Widget::widget('Widget_Abstract_Contents')->push($post);
                $post_likesNum = number_format($result['likesNum']);
                $post_title = htmlspecialchars($result['title']);
                $permalink = $result['permalink'];
                if($shownum == true){
                	echo "<li><a href='$permalink' title='$post_title'>$post_title</a><span style='font-size:70%'>$before $post_likesNum $after</span></li>\n";
              	}else{
              		echo "<li><a href='$permalink' title='$post_title  &#13;点赞数：$post_likesNum'>$post_title</a></li>\n";
              	}
            }

        } else {
            echo "<li>N/A</li>\n";
        }
    }



    /**
     * 输出评论最多的文章
     *
     * 语法: StatPro_Plugin::mostCommented();
     *
     * @access public
     * @param int     $limit  文章数目
     * @param string  $shownum 是否显示评论数量
     * @param string  $before 前字串
     * @param string  $after  后字串
     * @return string
     */
    public static function mostCommented($limit = 10, $shownum = false, $before = '<br/> - ( 评论: ', $after = ' 次 ) ')
    {
        $db = Typecho_Db::get();
        $options = Typecho_Widget::widget('Widget_Options');
        $limit = is_numeric($limit) ? $limit : 10;
        $posts = $db->fetchAll($db->select()->from('table.contents')
                 ->where('type = ? AND status = ? AND password IS NULL', 'post', 'publish')
                 ->order('commentsNum', Typecho_Db::SORT_DESC)
                 ->limit($limit)
                 );

        if ($posts) {
            foreach ($posts as $post) {
                $result = Typecho_Widget::widget('Widget_Abstract_Contents')->push($post);
                $post_commentsNum = number_format($result['commentsNum']);
                $post_title = htmlspecialchars($result['title']);
                $permalink = $result['permalink'];
                if($shownum == true){
                	echo "<li><a href='$permalink' title='$post_title'>$post_title</a><span style='font-size:70%'>$before $post_commentsNum $after</span></li>\n";
              	}else{
              		echo "<li><a href='$permalink' title='$post_title &#13;评论数：$post_commentsNum'>$post_title</a></li>\n";
              	}
            }

        } else {
            echo "<li>N/A</li>\n";
        }
    }
}